import json
import pathlib
import shutil
from pathlib import Path
from unittest.mock import Mock

import pytest
from pylsp import uris
from pylsp.config.config import Config
from pylsp.workspace import Workspace, Document


here = Path(__file__).parent
fixtures_dir = here / "fixtures"


@pytest.fixture
def config(workspace):
    """Return a config object."""
    cfg = Config(workspace.root_uri, {}, 0, {})
    cfg._plugin_settings = {
        "plugins": {
            "pylint": {
                "enabled": False,
                "args": [],
                "executable": None,
            },
        },
    }
    cfg._disabled_plugins = []
    return cfg


@pytest.fixture
def workspace(tmpdir):
    """Return a workspace."""
    ws = Workspace(uris.from_fs_path(str(tmpdir)), Mock())
    ws._config = Config(ws.root_uri, {}, 0, {})
    return ws


@pytest.fixture
def document(workspace):
    return create_document(workspace, "simple.py")


@pytest.fixture
def code_action_context():
    # https://microsoft.github.io/language-server-protocol/specifications/specification-3-17/#codeActionKind
    code_action_kind = [
        "",
        "quickfix",
        "refactor",
        "refactor.extract",
        "refactor.inline",
        "refactor.rewrite",
        "source",
        "source.organizeImports",
        "source.fixAll",
    ]

    return {
        "diagnostics": [],
        "only": code_action_kind,
    }


def create_document(workspace, name):
    template_path = fixtures_dir / name
    dest_path = Path(workspace.root_path) / name
    shutil.copy(template_path, dest_path)
    document_uri = uris.from_fs_path(str(dest_path))
    return Document(document_uri, workspace)


@pytest.fixture
def finding():
    file = pathlib.Path(fixtures_dir / "finding.json")
    with open(file) as f:
        return json.load(f)



from unittest.mock import ANY

from amazon_codeguru_jupyterlab_extension import plugin
from tests.conftest import *

def test_settings():
    result = plugin.pylsp_settings()
    expected = {
        "plugins": {}
    }
    assert result == expected


def test_lint_with_existing_diagnostics(config, workspace, document):
    diagnostics = [{
        "source": "foo",
        "code": "bar",
        "message": "baz"
    }]
    plugin.cfg.diagnostics_by_document = {
        document.uri: diagnostics
    }
    result = plugin.pylsp_lint(config, workspace, document, False)
    expected = diagnostics
    assert result == expected


def test_lint_without_existing_diagnostics(config, workspace, document):
    plugin.cfg.diagnostics_by_document = {}
    result = plugin.pylsp_lint(config, workspace, document, False)
    expected = None
    assert result == expected


def test_execute_run_scan(config, workspace, document):
    plugin.get_diagnostics_from_other_sources = Mock(return_value=[{"message": "1"}])
    plugin.get_diagnostics = Mock(return_value=[{"message": "2"}])
    plugin.Workspace.publish_diagnostics = Mock()
    plugin.execute_run_scan(config, workspace, document, "region")
    plugin.get_diagnostics_from_other_sources.assert_called_once_with(config, workspace, document)
    plugin.get_diagnostics.assert_called_once_with(workspace, document, "region")
    plugin.Workspace.publish_diagnostics.assert_called_once_with(doc_uri=document.uri, diagnostics=[{"message": "1"}, {"message": "2"}])


def test_execute_command_run_scan(config, workspace, document):
    plugin.execute_run_scan = Mock()
    workspace.get_document = Mock(return_value=document)
    plugin.pylsp_execute_command(config, workspace, "cgs.runScan", [document.uri, "region"])
    plugin.execute_run_scan.assert_called_once_with(config, workspace, document, "region")


def test_execute_command_other(config, workspace):
    plugin.execute_run_scan = Mock()
    plugin.pylsp_execute_command(config, workspace, "some-cmd", ["some", "args"])
    assert not plugin.execute_run_scan.called



import os
import boto3
import sys

CODEGURU_REGIONS = [
    'eu-north-1',
    'ap-southeast-2',
    'us-east-1',
    'us-east-2',
    'us-west-2',
    'ap-northeast-1',
    'ap-southeast-1',
    'eu-central-1',
    'eu-west-1',
    'eu-west-2'
]
DEFAULT_AWS_REGION = 'us-east-1'
session = boto3.Session()
session._loader.search_paths.extend([os.path.join(sys.prefix, "boto")])


def get_codeguru_security_client(overridden_region):
    aws_region = get_codeguru_supported_region(overridden_region)
    return session.client('codeguru-security',
                          endpoint_url="https://codeguru-security.{}.amazonaws.com".format(aws_region),
                          region_name=aws_region)


def get_codeguru_supported_region(overridden_region):
    aws_region = os.getenv("AWS_REGION")
    if is_codeguru_supported(aws_region) and overridden_region == DEFAULT_AWS_REGION:
        return aws_region
    return overridden_region or DEFAULT_AWS_REGION


def is_codeguru_supported(region):
    return region in set(CODEGURU_REGIONS)



def __CELL_EDGE__(x):
	pass
__CELL_EDGE__(2)
app.run(debug=True, host='127.0.1.1', port=5000, extra_files='csp.txt')

app = Flask('vulpy')
app.config['SECRET_KEY'] = 'aaaaaaa'


@app.route('/')
def do_home():
    return redirect('/posts')

@app.before_request
def before_request():
    g.session = libsession.load(request)

app.run(debug=True, host='127.0.1.1', ssl_context=('/tmp/acme.cert', '/tmp/acme.key'))
print('hello')
__CELL_EDGE__(0)
import json
import base64

from pathlib import Path

from flask import Flask, g, redirect, request

import libsession


def create(response, username):
    session = base64.b64encode(json.dumps({'username': username}).encode())
    response.set_cookie('vulpy_session', session)
    return response


def load(request):

    session = {}
    cookie = request.cookies.get('vulpy_session')

    try:
        if cookie:
            decoded = base64.b64decode(cookie.encode())
            if decoded:
                session = json.loads(base64.b64decode(cookie))
    except Exception:
        pass

    return session


def destroy(response):
    response.set_cookie('vulpy_session', '', expires=0)
    return response
__CELL_EDGE__(1)
app = Flask('vulpy')
app.config['SECRET_KEY'] = 'aaaaaaa'

csp_file = Path('csp.txt')
csp = ''

if csp_file.is_file():
    with csp_file.open() as f:
        for line in f.readlines():
            if line.startswith('#'):
                continue
            line = line.replace('\n', '')
            if line:
                csp += line
if csp:
    print('CSP:', csp)


@app.route('/')
def do_home():
    return redirect('/posts')

@app.before_request
def before_request():
    g.session = libsession.load(request)

@app.after_request
def add_csp_headers(response):
    if csp:
        response.headers['Content-Security-Policy'] = csp
    return response
__CELL_EDGE__(3)




import ast
import json
import sys
from pprint import pprint

def ipython2python(code):
    """Transform IPython syntax to pure Python syntax
    Parameters
    ----------
    code : str
        IPython code, to be transformed to pure Python
    """
    try:
        from IPython.core.inputtransformer2 import TransformerManager
    except ImportError:
        print(
            "IPython is needed to transform IPython syntax to pure Python."
            " Install ipython with `pip install -r requirements.txt`."
        )
        return code
    else:
        isp = TransformerManager()
        return isp.transform_cell(code)
commands = ['cd', 'ls', 'pip']
def is_command(line):
  for command in commands:
    if line.lstrip().startswith(command):
      return True
  return False
def run(inp, out):
  with open(inp,'r') as red:
    notebook = json.load(red)
    cells = notebook['cells']
    notebookScript = 'def __CELL_EDGE__(x):\n\tpass\n'
    cellMetaWithEC = [] # cells with execution count
    cellMetaWoEC = [] # cells without execution count
    cellnum = 0
    for cell in cells:
      if cell['cell_type'] == 'code':
        code = cell['source']
        codeStr = ""
        for c in code:
            codeStr+=c
        cellCode = ipython2python(codeStr)
        cellCode = cellCode.rstrip(cellCode[-1])# remove the last line break
        if len(code) == 1: # handle single line command
          if is_command(code[0]):
            cellCode = 'pass #' + cellCode
        if cell['execution_count'] is None:
          cellMetaWoEC.append((cellnum, -1, cellCode))
        else:
          cellMetaWithEC.append((cellnum, cell['execution_count'], cellCode))
        cellnum += 1
    print(str(len(cellMetaWithEC)) + " cells contain execution count.")
    print(str(len(cellMetaWoEC)) + " cells were not executed.")
    if len(cellMetaWithEC)>0:
        print("convert notebook cells in execution order.")
        cellMetaWithEC = sorted(cellMetaWithEC, key=lambda x: x[1])
    else:
        print("no executed cells was detected, convert notebook cells in linear order.")
    for cm in cellMetaWithEC: # append executed code cells.
        notebookScript += '__CELL_EDGE__('+str(cm[0])+')\n'+cm[2]+'\n'
    for cm in cellMetaWoEC: # append unexecuted code cells in the end.
        notebookScript += '__CELL_EDGE__('+str(cm[0])+')\n'+cm[2]+'\n'
    output = open(out,'w')
    output.write(notebookScript)
    output.close()

def main():
  if len(sys.argv) != 3:
    print('input must have 2 arguments\n\tinput file\n\toutput file\n')
  run(sys.argv[1], sys.argv[2])
  print("conversion done!")

if __name__ == '__main__':
  main()




from datetime import datetime
from enum import Enum
import logging
from functools import partial
import os
import re
import threading
import zipfile
import requests
from botocore.exceptions import ClientError
from pylsp import lsp
from pylsp.config.config import Config
from pylsp.workspace import Workspace, Document

from .constants import PLUGIN_NAME
from .codeguru import get_codeguru_security_client
from .parseNotebookIntoScript import run

logger = logging.getLogger(__name__)
TMP_DIR = "/tmp"


class CommandStatus(str, Enum):
    PENDING = "pending"
    ERROR = "error"
    COMPLETED = "completed"


def create_python_from_notebook(nb_filepath: str):
    py_filename = os.path.basename(nb_filepath).replace(".ipynb", ".py")
    py_filepath = os.path.join(TMP_DIR, py_filename)
    run(nb_filepath, py_filepath)
    return py_filepath


def create_zip_from_python(py_filepath: str):
    zip_filename = os.path.basename(py_filepath).replace(".py", ".zip")
    zip_filepath = os.path.join(TMP_DIR, zip_filename)
    with zipfile.ZipFile(zip_filepath, "w", zipfile.ZIP_DEFLATED) as zipf:
        zipf.write(py_filepath, arcname=os.path.basename(py_filepath))
    return zip_filepath


def upload_file(zip_filepath: str, scan_name: str, codeguru_security, send_notification):
    try:
        create_upload_url_response = codeguru_security.create_upload_url(
            fileName=os.path.basename(zip_filepath), scanName=scan_name)
    except ClientError as e:
        logger.error(e)
        send_notification({"status": CommandStatus.ERROR, "message": str(e)})
        return
    s3_url = create_upload_url_response['s3Url']
    request_headers = create_upload_url_response['requestHeaders']
    code_artifact_id = create_upload_url_response['codeArtifactId']

    with open(zip_filepath, "rb") as f:
        put_response = requests.put(s3_url, data=f, headers=request_headers)
        if not put_response.ok:
            logger.error("Upload failed")
            send_notification({"status": CommandStatus.ERROR,
                              "message": "File upload failed"})
            return

    return code_artifact_id


def create_scan(code_artifact_id: str, scan_name: str, codeguru_security, send_notification):
    try:
        create_scan_response = codeguru_security.create_scan(resourceId={"codeArtifactId": code_artifact_id},
                                                             scanName=scan_name,
                                                             scanType="Express",
                                                             analysisType="All")
    except ClientError as e:
        logger.error(e)
        send_notification({"status": CommandStatus.ERROR, "message": str(e)})
        return
    run_id = create_scan_response['runId']
    return run_id


def poll_scan_status(scan_name: str, run_id: str, codeguru_security, send_notification):
    e = threading.Event()
    while not e.wait(5):
        try:
            get_scan_response = codeguru_security.get_scan(
                scanName=scan_name, runId=run_id)
        except ClientError as e:
            logger.error(e)
            send_notification(
                {"status": CommandStatus.ERROR, "message": str(e)})
            return
        if get_scan_response['scanState'] == 'Successful':
            break
        elif get_scan_response['scanState'] == 'Failed':
            logger.error("Express scan failed")
            send_notification({"status": CommandStatus.ERROR, "message": "Scan failed"})
            return


def get_scan_findings(scan_name: str, codeguru_security, send_notification):
    try:
        get_findings_response = codeguru_security.get_findings(
            scanName=scan_name)
    except ClientError as e:
        logger.error(e)
        send_notification({"status": CommandStatus.ERROR, "message": str(e)})
        return
    findings = get_findings_response['findings']
    send_notification(
        {"status": CommandStatus.COMPLETED, "message": len(findings)})
    return findings


def get_cell_md(py_filepath):
    # cell_md is a 2D array where each inner array has the following structure
    # [cell_number, range_start, range_end, cumulative_length, final_offset]
    # cell_number: 0-index cell number, which may not be sequential depending on execution order
    # range_start: line number (inclusive) in .py file which the cell range starts
    # range_end: line number (inclusive) in .py file which the cell range ends
    # cumulative_length: number of lines of the current cell plus all the cells before it in the original order
    # final_offset: the number to add to .py line number to get the .ipynb line
    cell_md = []
    with open(py_filepath) as f:
        for i, line in enumerate(f):
            m = re.match("__CELL_EDGE__\((.+)\)", line)
            if m and m.group(1):
                if len(cell_md) - 1 >= 0:
                    cell_md[len(cell_md) - 1].append(i)
                cell_number = int(m.group(1))
                cell_md.append([cell_number, i + 2])
        cell_md[len(cell_md) - 1].append(i + 2)
        cell_md.sort(key=lambda cell_md: cell_md[0])

        for i, c in enumerate(cell_md):
            c.append(c[2] - c[1] + (cell_md[i - 1][3] if i > 0 else 0))
            c.append(cell_md[i - 1][3] + 3 * c[0] - c[1] if i > 0 else -c[1])

        return cell_md


def get_message_for_finding(finding, py_filepath, is_nb_file):
    start_line = finding['vulnerability']['filePath']['startLine']
    end_line = finding['vulnerability']['filePath']['endLine']
    message = finding['remediation']['recommendation']['text']
    detector_id = finding['detectorId']
    detector_name = finding['detectorName']
    severity = finding['severity']

    with open(py_filepath) as f:
        lines = f.readlines()
        start_char = len(lines[start_line - 1]) - \
            len(lines[start_line - 1].lstrip())
        end_char = len(lines[end_line - 1])

    offset = -1
    if is_nb_file:
        cell_md = get_cell_md(py_filepath)
        for c in cell_md:
            if start_line >= c[1] and start_line <= c[2]:
                offset = c[4]
                break
    actual_start_line = start_line + offset
    actual_end_line = end_line + offset

    return {
        "source": "codeguru-security",
        "code": detector_id,
        "message": "Issue: {detector_name}\n\nSuggested remediation: {remediation}".format(detector_name=detector_name, remediation=message),
        "range": {
            "start": {"line": actual_start_line, "character": start_char},
            "end": {"line": actual_end_line, "character": end_char}
        },
        "severity": lsp.DiagnosticSeverity.Information if severity == 'Info' else lsp.DiagnosticSeverity.Warning
    }


def get_diagnostics(workspace: Workspace, document: Document, overridden_region: str):
    codeguru_security = get_codeguru_security_client(overridden_region)

    with workspace.report_progress("command: runScan") as send_notification:
        send_notification({"status": CommandStatus.PENDING})
        is_nb_file = document.filename.endswith(".ipynb")
        path_to_nb = document.path.replace(".virtual_documents/", "")
        py_filepath = create_python_from_notebook(
            path_to_nb) if is_nb_file else document.path
        zip_filepath = create_zip_from_python(py_filepath)
        scan_name = "{}-{}".format(os.path.basename(zip_filepath),
                                   datetime.now().isoformat())
        code_artifact_id = upload_file(
            zip_filepath, scan_name, codeguru_security, send_notification)
        run_id = create_scan(code_artifact_id, scan_name,
                             codeguru_security, send_notification)
        poll_scan_status(scan_name, run_id,
                         codeguru_security, send_notification)
        findings = get_scan_findings(
            scan_name, codeguru_security, send_notification)
        return list(map(partial(get_message_for_finding, py_filepath=py_filepath, is_nb_file=is_nb_file), findings))


def flatten(list_of_lists):
    return [item for lst in list_of_lists for item in lst]


def get_diagnostics_from_other_sources(config: Config, workspace: Workspace, document: Document):
    plugin = config.plugin_manager.get_plugin(PLUGIN_NAME)
    hook_handlers = config.plugin_manager.subset_hook_caller(
        'pylsp_lint', config.disabled_plugins + [plugin])
    return flatten(hook_handlers(config=config, workspace=workspace, document=document, is_saved=False))



import os
from unittest import mock
from botocore.stub import Stubber

from amazon_codeguru_jupyterlab_extension import diagnostics
from tests.conftest import *


def test_get_cell_md():
    result = diagnostics.get_cell_md(fixtures_dir / "converted.py")
    expected = [
        [0, 21, 55, 34, -21],
        [1, 57, 87, 64, -20],
        [2, 4, 19, 79, 66],
        [3, 89, 90, 80, -1]
    ]
    assert result == expected


def test_get_message_for_finding_for_nb(finding):
    result = diagnostics.get_message_for_finding(finding, fixtures_dir / "converted.py", True)
    expected = {
        "code": "python/improper-error-handling@v1.0",
        "message": "Issue: Improper error handling\n\nSuggested remediation: Try, Except, Pass detected. https://bandit.readthedocs.io/en/latest/plugins/b110_try_except_pass.html",
        "range": {
            "end": { "character": 22, "line": 26 },
            "start": { "character": 4, "line": 26 }
        },
        "severity": 2,
        "source": "codeguru-security"
    }
    assert result == expected


def test_flatten():
    result = diagnostics.flatten([[1, 2, 3], [4], [5, 6]])
    expected = [1, 2, 3, 4, 5, 6]
    assert result == expected


def test_create_python_from_notebook(workspace):
    document = create_document(workspace, "simple.ipynb")
    result = diagnostics.create_python_from_notebook(document.path)
    expected_path = "/tmp/simple.py"
    with open(expected_path) as f:
        lines = f.readlines()
    assert lines == [
        "def __CELL_EDGE__(x):\n",
        "\tpass\n",
        "__CELL_EDGE__(0)\n",
        "import sys\n",
        "def main():\n",
        "    print(sys.stdin.read())\n"
    ]
    assert result == expected_path


def test_create_zip_from_python(workspace):
    document = create_document(workspace, "simple.py")
    result = diagnostics.create_zip_from_python(document.path)
    expected_path = "/tmp/simple.zip"
    assert os.path.isfile(expected_path)
    assert result == expected_path


def test_get_diagnostics_from_other_sources(config, workspace, document):
    mock_hook_handlers = Mock(return_value=[[{"message": "other"}]])
    config.plugin_manager.subset_hook_caller = Mock(return_value=mock_hook_handlers)
    result = diagnostics.get_diagnostics_from_other_sources(config, workspace, document)
    expected = [{"message": "other"}]
    assert result == expected


def test_create_scan():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    create_scan_response = {
        "runId": "123",
        "scanName": "scan-name",
        "resourceId": {"codeArtifactId": "code-artifact-id"},
        "scanState": "Pending"
    }
    expected_params = {
        "resourceId": {"codeArtifactId": "code-artifact-id"},
        "scanName": "scan-name",
        "scanType": "Express",
        "analysisType": "All"
    }
    with Stubber(client) as stubber:
        stubber.add_response("create_scan", create_scan_response, expected_params)
        send_notification = Mock()
        result = diagnostics.create_scan("code-artifact-id", "scan-name", client, send_notification)
        assert result == "123"


def test_create_scan_failed():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    with Stubber(client) as stubber:
        stubber.add_client_error("create_scan", service_error_code="foo", service_message="bar")
        send_notification = Mock()
        diagnostics.create_scan("code-artifact-id", "scan-name", client, send_notification)
        send_notification.assert_called_once_with({
            "message": "An error occurred (foo) when calling the CreateScan operation: bar",
            "status": "error",
        })


@mock.patch('requests.put')
def test_upload_file(mock_put):
    client = diagnostics.get_codeguru_security_client("us-west-2")
    create_upload_url_response = {
        "s3Url": "https://s3-url.com",
        "requestHeaders": {"a": "b", "c": "d"},
        "codeArtifactId": "code-artifact-id"
    }
    create_upload_url_expected_params = {
        "fileName": "simple.zip",
        "scanName": "scan-name"
    }
    with Stubber(client) as stubber:
        stubber.add_response("create_upload_url", create_upload_url_response, create_upload_url_expected_params)
        send_notification = Mock()
        result = diagnostics.upload_file("/tmp/simple.zip", "scan-name", client, send_notification)
        mock_put.assert_called_once_with("https://s3-url.com", data=mock.ANY, headers={"a": "b", "c": "d"})
        assert result == "code-artifact-id"


def test_upload_file_create_upload_url_failed():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    with Stubber(client) as stubber:
        stubber.add_client_error("create_upload_url", service_error_code="foo", service_message="bar")
        send_notification = Mock()
        diagnostics.upload_file("/tmp/simple.zip", "scan-name", client, send_notification)
        send_notification.assert_called_once_with({
            "message": "An error occurred (foo) when calling the CreateUploadUrl operation: bar",
            "status": "error",
        })


@mock.patch('requests.put')
def test_upload_file_failed(mock_put):
    client = diagnostics.get_codeguru_security_client("us-west-2")
    create_upload_url_response = {
        "s3Url": "https://s3-url.com",
        "requestHeaders": {"a": "b", "c": "d"},
        "codeArtifactId": "code-artifact-id"
    }
    with Stubber(client) as stubber:
        stubber.add_response("create_upload_url", create_upload_url_response)
        send_notification = Mock()
        mock_put.return_value.ok = False
        diagnostics.upload_file("/tmp/simple.zip", "scan-name", client, send_notification)
        mock_put.assert_called_once_with("https://s3-url.com", data=mock.ANY, headers={"a": "b", "c": "d"})
        send_notification.assert_called_once_with({
            "message": "File upload failed",
            "status": "error",
        })


def test_get_scan_findings(finding):
    client = diagnostics.get_codeguru_security_client("us-west-2")
    get_findings_response = {
        "findings": [finding]
    }
    get_findings_expected_params = {
        "scanName": "scan-name"
    }
    with Stubber(client) as stubber:
        stubber.add_response("get_findings", get_findings_response, get_findings_expected_params)
        send_notification = Mock()
        result = diagnostics.get_scan_findings("scan-name", client, send_notification)
        send_notification.assert_called_once_with({
            "message": 1,
            "status": "completed"
        })
        assert result == [finding]


def test_get_scan_findings_failed():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    with Stubber(client) as stubber:
        stubber.add_client_error("get_findings", service_error_code="foo", service_message="bar")
        send_notification = Mock()
        diagnostics.get_scan_findings("scan-name", client, send_notification)
        send_notification.assert_called_once_with({
            "message": "An error occurred (foo) when calling the GetFindings operation: bar",
            "status": "error"
        })


def test_poll_scan_status_succeeded():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    get_scan_response = {
        "scanState": "Successful",
        "scanName": "scan-name",
        "runId": "run-id",
        "createdAt": 1,
        "analysisType": "All"
    }
    get_scan_expected_params = {
        "scanName": "scan-name",
        "runId": "run-id"
    }
    with Stubber(client) as stubber:
        stubber.add_response("get_scan", get_scan_response, get_scan_expected_params)
        send_notification = Mock()
        diagnostics.poll_scan_status("scan-name", "run-id", client, send_notification)
        send_notification.assert_not_called()


def test_poll_scan_status_failed():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    get_scan_response = {
        "scanState": "Failed",
        "scanName": "scan-name",
        "runId": "run-id",
        "createdAt": 1,
        "analysisType": "All"
    }
    get_scan_expected_params = {
        "scanName": "scan-name",
        "runId": "run-id"
    }
    with Stubber(client) as stubber:
        stubber.add_response("get_scan", get_scan_response, get_scan_expected_params)
        send_notification = Mock()
        diagnostics.poll_scan_status("scan-name", "run-id", client, send_notification)
        send_notification.assert_called_once_with({
            "message": "Scan failed",
            "status": "error"
        })


def test_poll_scan_status_get_scan_failed():
    client = diagnostics.get_codeguru_security_client("us-west-2")
    with Stubber(client) as stubber:
        stubber.add_client_error("get_scan", service_error_code="foo", service_message="bar")
        send_notification = Mock()
        diagnostics.poll_scan_status("scan_name", "run-id", client, send_notification)
        send_notification.assert_called_once_with({
            "message": "An error occurred (foo) when calling the GetScan operation: bar",
            "status": "error"
        })


def test_get_message_for_finding_nb():
    finding = {
        "detectorId": "python/foo@v1.0",
        "detectorName": "Foo",
        "remediation": {
            "recommendation": {
                "text": "Bar"
            }
        },
        "severity": "Low",
        "vulnerability": {
            "filePath": {
                "startLine": 4,
                "endLine": 4
            }
        }
    }
    result = diagnostics.get_message_for_finding(finding, "/tmp/simple.py", True)
    assert result == {
        "source": "codeguru-security",
        "code": "python/foo@v1.0",
        "message": "Issue: Foo\n\nSuggested remediation: Bar",
        "range": {
            "start": {"line": 0, "character": 0},
            "end": {"line": 0, "character": 11}
        },
        "severity": 2
    }


def test_get_message_for_finding_python():
    finding = {
        "detectorId": "python/foo@v1.0",
        "detectorName": "Foo",
        "remediation": {
            "recommendation": {
                "text": "Bar"
            }
        },
        "severity": "Low",
        "vulnerability": {
            "filePath": {
                "startLine": 4,
                "endLine": 4
            }
        }
    }
    result = diagnostics.get_message_for_finding(finding, "/tmp/simple.py", False)
    assert result == {
        "source": "codeguru-security",
        "code": "python/foo@v1.0",
        "message": "Issue: Foo\n\nSuggested remediation: Bar",
        "range": {
            "start": {"line": 3, "character": 0},
            "end": {"line": 3, "character": 11}
        },
        "severity": 2
    }




import logging
import os
from typing import List
from pylsp import hookimpl
from pylsp.config.config import Config
from pylsp.workspace import Workspace, Document
from . import cfg
from .constants import DEFAULT_LOG_LOCATION, LOG_NAME, PLUGIN_NAME
from .diagnostics import get_diagnostics, get_diagnostics_from_other_sources

logging.basicConfig(
    filename=os.path.join(DEFAULT_LOG_LOCATION, LOG_NAME),
    filemode="a",
    force=True,
    format="%(asctime)s.%(msecs)d %(name)s %(levelname)s %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    level=logging.INFO)

logger = logging.getLogger(__name__)


@hookimpl
def pylsp_settings():
    logger.info("Initializing {}".format(PLUGIN_NAME))

    return {
        "plugins": {},
    }


@hookimpl
def pylsp_execute_command(config: Config, workspace: Workspace, command: str, arguments: List[str]):
    logger.info("workspace/executeCommand: %s %s", command, arguments)

    if command == "cgs.runScan":
        doc_uri = arguments[0]
        overridden_region = arguments[1]
        document = workspace.get_document(doc_uri)
        execute_run_scan(config, workspace, document, overridden_region)


@hookimpl
def pylsp_lint(config: Config, workspace: Workspace, document: Document, is_saved: bool):
    return cfg.diagnostics_by_document.get(document.uri)


def execute_run_scan(config: Config, workspace: Workspace, document: Document, overridden_region: str):
    other_diagnostics = get_diagnostics_from_other_sources(config, workspace, document)
    cfg.diagnostics_by_document[document.uri] = get_diagnostics(workspace, document, overridden_region)
    workspace.publish_diagnostics(doc_uri=document.uri, diagnostics=other_diagnostics + cfg.diagnostics_by_document.get(document.uri))



import os

from setuptools import setup

# Declare your non-python data files:
# Files underneath configuration/ will be copied into the build preserving the
# subdirectory structure if they exist.
data_files = []
for root, dirs, files in os.walk("configuration"):
    data_files.append(
        (os.path.relpath(root, "configuration"), [os.path.join(root, f) for f in files])
    )

setup(
    # include data files
    data_files=data_files,
)
